package ai.pai.lensman.main;

import android.os.AsyncTask;
import android.os.SystemClock;
import android.support.annotation.StringRes;

import com.android.common.executors.ThreadExecutor;
import com.android.common.http.HttpUtils;

import org.json.JSONObject;

import java.util.HashMap;

import ai.pai.lensman.BuildConfig;
import ai.pai.lensman.R;
import ai.pai.lensman.base.BaseInteractor;
import ai.pai.lensman.utils.BoxUrlContainer;
import ai.pai.lensman.utils.HttpPostTask;
import ai.pai.lensman.utils.UrlContainer;

/**
 * Created by chengzhenyu on 2017/4/4.
 */

public class SyncTimeInteractor implements BaseInteractor {
    private HttpPostTask timeSyncTask;
    private String lensmanId;
    private SyncTimeListener listener;
    private boolean isCancelled;

    public SyncTimeInteractor(String lensmanId,  SyncTimeListener listener){
        this.listener = listener;
        this.lensmanId = lensmanId;
    }

    @Override
    public void startJob() {
        cancelJob();
        isCancelled = false;
        HashMap<String,String> params = new HashMap<>();
        params.put("user_id",lensmanId);
        if(BuildConfig.isTestMode){
            listener.onTimeSync(true,0);
        }
        timeSyncTask = new HttpPostTask(params){

            int msgId = R.string.sync_time_box_error;

            @Override
            protected boolean parseResponse(String response) {
                try{
                    JSONObject json = new JSONObject(response);
                    int status = json.getInt("status");
                    if(status == 200){
                        JSONObject data = json.getJSONObject("data");
                        String time = data.getString("time");
                        HashMap<String,String> boxParams = new HashMap<>();
                        boxParams.put("time",time);
                        if(isCancelled){
                            return false;
                        }
                        HttpUtils.doHttpPost(BoxUrlContainer.SET_TIME_URL,boxParams);
                        if(isCancelled){
                            return false;
                        }
                        SystemClock.sleep(1000);
                        if(isCancelled){
                            return false;
                        }
                        String boxResp = HttpUtils.doHttpPost(BoxUrlContainer.BOX_TIME_URL,null);
                        if(isCancelled){
                            return false;
                        }
                        JSONObject boxJson = new JSONObject(boxResp);
                        int boxStatus = boxJson.getInt("status");
                        if(boxStatus == 200){
                            String boxTime = boxJson.getJSONObject("data").getString("time");
                            if(time.substring(0,13).equals(boxTime.substring(0,13))){
                                return true;
                            }
                        }
                    }else{
                        msgId = R.string.sync_time_server_error;
                    }
                }catch (Exception e){
                    e.printStackTrace();
                }
                return false;
            }

            @Override
            protected void onPostFail() {
                super.onPostFail();
                listener.onTimeSync(false,msgId);
            }

            @Override
            protected void onPostSuccess() {
                super.onPostSuccess();
                listener.onTimeSync(true,0);
            }
        };
        timeSyncTask.executeOnExecutor(ThreadExecutor.getInstance().getExecutor(), UrlContainer.SERVER_TIME_URL);
    }

    @Override
    public void cancelJob() {
        if(timeSyncTask==null){
            return;
        }
        if(timeSyncTask.getStatus()== AsyncTask.Status.RUNNING){
            timeSyncTask.cancel(true);
        }
        isCancelled = true;
        timeSyncTask = null;
    }

    public interface SyncTimeListener{

        void onTimeSync(boolean result,@StringRes int strId);
    }
}
